=== modified file 'errors.py'
--- errors.py	2006-02-06 14:04:14 +0000
+++ errors.py	2006-03-12 14:12:00 +0000
@@ -3,3 +3,6 @@
 except ImportError:
     class CommandError(Exception):
         pass
+
+class PatchFailed(Exception):
+    """Failed applying patch!"""

=== modified file 'shelf.py'
--- shelf.py	2006-03-12 13:57:03 +0000
+++ shelf.py	2006-03-12 14:12:00 +0000
@@ -4,7 +4,7 @@
 import sys
 import subprocess
 from datetime import datetime
-from errors import CommandError
+from errors import CommandError, PatchFailed
 from hunk_selector import ShelveHunkSelector, UnshelveHunkSelector
 from patchsource import PatchSource, FilePatchSource
 
@@ -157,8 +157,12 @@
         try:
             self._run_patch(to_unshelve, dry_run=True)
             self._run_patch(to_unshelve)
-        except CommandError:
-            raise CommandError("Your shelved patch no " \
+        except PatchFailed:
+            try:
+                self._run_patch(to_unshelve, strip=0, dry_run=True)
+                self._run_patch(to_unshelve, strip=0)
+            except PatchFailed:
+                raise CommandError("Your shelved patch no " \
                     "longer applies cleanly to the working tree!")
 
         # Backup the shelved patch
@@ -205,8 +209,8 @@
 
         self._run_patch(to_shelve, reverse=True)
 
-    def _run_patch(self, patches, reverse=False, dry_run=False):
-        args = ['patch', '-d', self.base, '-s', '-p1', '-f']
+    def _run_patch(self, patches, strip=1, reverse=False, dry_run=False):
+        args = ['patch', '-d', self.base, '-s', '-p%d' % strip, '-f']
         if reverse:
             args.append('-R')
         if dry_run:
@@ -224,7 +228,7 @@
 
         result = process.wait()
         if result != 0:
-            raise CommandError("Failed applying patches!")
+            raise PatchFailed()
 
         return result
 

=== modified file 'tests.py'
--- tests.py	2006-03-12 13:55:53 +0000
+++ tests.py	2006-03-12 14:12:00 +0000
@@ -341,3 +341,24 @@
 
         # Shelve should work now
         self.run_bzr('shelve', '--all', retcode=0)
+
+    def test_shelf_p0_patch(self):
+        tree = self.make_branch_and_tree('.')
+
+        self.__create_and_add_test_file(tree)
+
+        # Run a benign shelf command to setup .shelf for us
+        self.run_bzr('shelf', 'ls', retcode=0)
+
+        # Fake a -p0 shelved patch
+        diff = self.DIFF_1
+        diff = diff.replace('a/', '')
+        diff = diff.replace('b/', '')
+        open('.shelf/shelves/default/00', 'w').write(diff)
+
+        # This should work
+        self.run_bzr('unshelve', '--all', retcode=0)
+
+        # Check the diff is right
+        self.assertEqual(self.capture('diff', retcode=1),
+            self.DIFF_HEADER + self.DIFF_1 + '\n')

